'''Plot load lines

Overlapped elliptical load lines with φ = 0°, 30°, 45°, 60°, and 85° when
R is always 6.4Ω.
'''
import numpy as np
import matplotlib as mp
import matplotlib.pyplot as pp

def main():
  # reactive load angles
  phi = np.radians(np.array([0.0, 30.0, 45.0, 60.0, 70.0, 85.0]))
  # minimum resistance of load
  R = 6.4
  # supply voltage
  Vcc = 45.0
  # peak signal voltage on load (my guess)
  Vp = Vcc - 4.0
  # signal phase
  gamma = np.linspace(-0.5*np.pi, 0.5*np.pi, 200)
  # calculate and plot load lines for each value of phi
  for psi in phi:
    # calculate load line coordinates
    Vce = Vcc - Vp*np.cos(gamma+psi)
    Ic = (Vp/R)*np.cos(psi)*np.cos(gamma)
    # calculate coordinates with maximum Ic on load line
    Vce_max = Vcc - Vp*np.cos(psi)
    Ic_max = (Vp/R) *np.cos(psi)
    # plot and annotate load line
    if psi == 0.0:
      plot_color = 'green'
      plot_linewidth = '1.0'
    else:
      plot_color = 'blue'
      plot_linewidth = '0.6'
    pp.plot(Vce, Ic, linewidth=plot_linewidth, color=plot_color)
    pp.annotate("{:.0f}°".format(np.degrees(psi)), (Vce_max, Ic_max), (1, 4),
          textcoords='offset points', fontstyle='oblique', color=plot_color)
  # plot load line limit
  V_max = np.array([Vcc-Vp, Vcc+Vp])
  I_max = np.array([Vp/R, 0.0])
  pp.plot(V_max, I_max, color='red', linewidth=2)
  # create a useful legend
  green_line = mp.lines.Line2D([], [], linewidth=1.0, color='green',
      label='Resistance only')
  blue_line = mp.lines.Line2D([], [], linewidth=0.5, color='blue',
      label='Reactive loads')
  red_line = mp.lines.Line2D([], [], linewidth= 2, color='red',
      label='Load line limit')
  pp.legend(handles=[green_line, blue_line, red_line])
  # label the axis, add grid and title
  pp.xlabel('Vce (V)')
  pp.ylabel('Ic (A)')
  pp.grid(True)
  pp.title('Load Lines (R=6.4Ω)')
  # show it to me
  pp.show(block=True)

if __name__ == '__main__':
  main()
